/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2024 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/*

This code is for a homemade STM32-based USB keypad with multimedia buttons by Nanoslavic
My english YouTube https://www.youtube.com/@NanoslavicLab

Этот код для самодельной USB клавиатуры с медиа кнопками на основе STM32 от Нанославика
Мой русский канал YouTube https://www.youtube.com/@nanoslavic

*/

/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "usb_device.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

#include "key_names.h"
#include "usbd_customhid.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
IWDG_HandleTypeDef hiwdg;

/* USER CODE BEGIN PV */
extern USBD_HandleTypeDef hUsbDeviceFS;
extern const uint8_t KeyMap[];


/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_IWDG_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

uint8_t report_buf[9];

void ClearBuf(void){
	for (int i=0; i<9; i++){
		report_buf[i]=0;
	}
}

//Press key
void SendKey(uint8_t modifier, uint8_t key){
	ClearBuf();
	report_buf[0] = 1;
	report_buf[1] = modifier;
	report_buf[3] = key;

	int time_out = 0;
	while (USBD_CUSTOM_HID_SendReport(&hUsbDeviceFS,report_buf, 9u) != USBD_OK){
		HAL_Delay(2);
		time_out++;
		if (time_out>50) return;
		wdr();
	};
}

void SendMediaKey(uint8_t key){
	ClearBuf();
	report_buf[0] = 2;
	report_buf[1] = key;
	report_buf[2] = 0;
	int time_out = 0;
	while (USBD_CUSTOM_HID_SendReport(&hUsbDeviceFS,report_buf, 3u) != USBD_OK){
		HAL_Delay(2);
		time_out++;
		if (time_out>50) return;
		wdr();
	};
}

void CheckAndPress(GPIO_TypeDef* GPIOx, uint16_t GPIO_Pin, uint8_t modifier, uint8_t key){
	if (HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_SET){
		wdr();
		HAL_Delay(1);
		if (HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_RESET) return;//Noise canceling
		HAL_Delay(1);
		if (HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_RESET) return;//Noise canceling

		HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);//Led on
		wdr();
		//Press key
		if (key > 0xDF){
			//Media key
			SendMediaKey(1 << (key - 0xE0));
		} else {
			SendKey(modifier,key);
		}

		wdr();
		HAL_Delay(20);

		while (HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_SET){
			wdr();
			HAL_Delay(1);
		}

		//Release key
		if (key > 0xDF){
			//Media key
			SendMediaKey(0);
		} else {
			SendKey(0,0);
		}
		wdr();
		HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_SET);//Led off
		HAL_Delay(15);
		wdr();
	}
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USB_DEVICE_Init();
  MX_IWDG_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  //Led blinking
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);
  HAL_Delay(100);
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_SET);
  HAL_Delay(100);
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);
  HAL_Delay(100);
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_SET);
  HAL_Delay(100);

  while (1)
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
	  wdr();//Reset Watchdog timer

	  //Read keys and send codes to PC
	  CheckAndPress(GPIOC,	GPIO_PIN_14,	KeyMap[0],	KeyMap[1]);		//Key 1
	  CheckAndPress(GPIOC,	GPIO_PIN_15,	KeyMap[2],	KeyMap[3]);		//Key 2
	  CheckAndPress(GPIOA,	GPIO_PIN_0,		KeyMap[4],	KeyMap[5]);		//Key 3
	  CheckAndPress(GPIOA,	GPIO_PIN_1,		KeyMap[6],	KeyMap[7]);		//Key 4
	  CheckAndPress(GPIOA,	GPIO_PIN_2,		KeyMap[8],	KeyMap[9]);		//Key 5
	  CheckAndPress(GPIOA,	GPIO_PIN_3,		KeyMap[10],	KeyMap[11]);	//Key 6
	  CheckAndPress(GPIOA,	GPIO_PIN_4,		KeyMap[12],	KeyMap[13]);	//Key 7
	  CheckAndPress(GPIOA,	GPIO_PIN_5,		KeyMap[14],	KeyMap[15]);	//Key 8
	  CheckAndPress(GPIOA,	GPIO_PIN_6,		KeyMap[16],	KeyMap[17]);	//Key 9
	  CheckAndPress(GPIOA,	GPIO_PIN_7,		KeyMap[18],	KeyMap[19]);	//Key 10
	  CheckAndPress(GPIOB,	GPIO_PIN_0,		KeyMap[20],	KeyMap[21]);	//Key 11
	  CheckAndPress(GPIOB,	GPIO_PIN_1,		KeyMap[22],	KeyMap[23]);	//Key 12
	  CheckAndPress(GPIOB,	GPIO_PIN_2,		KeyMap[24],	KeyMap[25]);	//Key 13
	  CheckAndPress(GPIOB,	GPIO_PIN_10,	KeyMap[26],	KeyMap[27]);	//Key 14
	  CheckAndPress(GPIOB,	GPIO_PIN_12,	KeyMap[28],	KeyMap[29]);	//Key 15
	  CheckAndPress(GPIOB,	GPIO_PIN_13,	KeyMap[30],	KeyMap[31]);	//Key 16
	  CheckAndPress(GPIOB,	GPIO_PIN_14,	KeyMap[32],	KeyMap[33]);	//Key 17
	  CheckAndPress(GPIOB,	GPIO_PIN_15,	KeyMap[34],	KeyMap[35]);	//Key 18
	  CheckAndPress(GPIOA,	GPIO_PIN_8,		KeyMap[36],	KeyMap[37]);	//Key 19
	  CheckAndPress(GPIOA,	GPIO_PIN_9,		KeyMap[38],	KeyMap[39]);	//Key 20
	  CheckAndPress(GPIOA,	GPIO_PIN_10,	KeyMap[40],	KeyMap[41]);	//Key 21
	  CheckAndPress(GPIOA,	GPIO_PIN_15,	KeyMap[42],	KeyMap[43]);	//Key 22
	  CheckAndPress(GPIOB,	GPIO_PIN_3,		KeyMap[44],	KeyMap[45]);	//Key 23
	  CheckAndPress(GPIOB,	GPIO_PIN_4,		KeyMap[46],	KeyMap[47]);	//Key 24
	  CheckAndPress(GPIOB,	GPIO_PIN_5,		KeyMap[48],	KeyMap[49]);	//Key 25
	  CheckAndPress(GPIOB,	GPIO_PIN_6,		KeyMap[50],	KeyMap[51]);	//Key 26
	  CheckAndPress(GPIOB,	GPIO_PIN_7,		KeyMap[52],	KeyMap[53]);	//Key 27
	  CheckAndPress(GPIOB,	GPIO_PIN_8,		KeyMap[54],	KeyMap[55]);	//Key 28
	  CheckAndPress(GPIOB,	GPIO_PIN_9,		KeyMap[56],	KeyMap[57]);	//Key 29

  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief IWDG Initialization Function
  * @param None
  * @retval None
  */
static void MX_IWDG_Init(void)
{

  /* USER CODE BEGIN IWDG_Init 0 */

  /* USER CODE END IWDG_Init 0 */

  /* USER CODE BEGIN IWDG_Init 1 */

  /* USER CODE END IWDG_Init 1 */
  hiwdg.Instance = IWDG;
  hiwdg.Init.Prescaler = IWDG_PRESCALER_4;
  hiwdg.Init.Reload = 4095;
  if (HAL_IWDG_Init(&hiwdg) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN IWDG_Init 2 */

  /* USER CODE END IWDG_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
/* USER CODE BEGIN MX_GPIO_Init_1 */
/* USER CODE END MX_GPIO_Init_1 */

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);

  /*Configure GPIO pin : PC13 */
  GPIO_InitStruct.Pin = GPIO_PIN_13;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PC14 PC15 */
  GPIO_InitStruct.Pin = GPIO_PIN_14|GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PA0 PA1 PA2 PA3
                           PA4 PA5 PA6 PA7
                           PA8 PA9 PA10 PA15 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2|GPIO_PIN_3
                          |GPIO_PIN_4|GPIO_PIN_5|GPIO_PIN_6|GPIO_PIN_7
                          |GPIO_PIN_8|GPIO_PIN_9|GPIO_PIN_10|GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PB0 PB1 PB2 PB10
                           PB12 PB13 PB14 PB15
                           PB3 PB4 PB5 PB6
                           PB7 PB8 PB9 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2|GPIO_PIN_10
                          |GPIO_PIN_12|GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15
                          |GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5|GPIO_PIN_6
                          |GPIO_PIN_7|GPIO_PIN_8|GPIO_PIN_9;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

/* USER CODE BEGIN MX_GPIO_Init_2 */
/* USER CODE END MX_GPIO_Init_2 */
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
